@@ -19,7 +19,7 @@ namespace math {
  * defined by applying a functor to a combination of scalars,
  * containers of matching sizes, or a combination of a scalar and a container.
  * These containers can be a standard library vector, Eigen dense
- * matrix expression template, or container of these. For each specialisation,
+ * matrix expression template, or container of these. For each specialization,
  * the same type as the largest (dimension) input is returned.
  *
  * This base template function takes (and returns) scalars.
@@ -39,7 +39,7 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 }
 
 /**
- * Specialisation for use with two Eigen inputs. Eigen's binaryExpr framework
+ * Specialization for use with two Eigen inputs. Eigen's binaryExpr framework
  * is used for more efficient indexing of both row- and column-major inputs
  * without separate loops.
  *
@@ -53,17 +53,17 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
  */
 template <typename T1, typename T2, typename F,
           require_all_eigen_t<T1, T2>* = nullptr>
-inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
+inline auto apply_scalar_binary(T1&& x, T2&& y, F&& f) {
   check_matching_dims("Binary function", "x", x, "y", y);
-#ifdef USE_STANC3
-  return x.binaryExpr(y, f);
-#else
-  return x.binaryExpr(y, f).eval();
-#endif
+  return make_holder(
+      [](auto& f_inner, auto& x_inner, auto& y_inner) {
+        return x_inner.binaryExpr(y_inner, f_inner);
+      },
+      std::forward<F>(f), std::forward<T1>(x), std::forward<T2>(y));
 }
 
 /**
- * Specialisation for use with one Eigen vector (row or column) and
+ * Specialization for use with one Eigen vector (row or column) and
  * a one-dimensional std::vector of integer types
  *
  * @tparam T1 Type of first argument to which functor is applied.
@@ -77,19 +77,20 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 template <typename T1, typename T2, typename F,
           require_eigen_vector_vt<is_stan_scalar, T1>* = nullptr,
           require_std_vector_vt<std::is_integral, T2>* = nullptr>
-inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
+inline auto apply_scalar_binary(T1&& x, T2&& y, F&& f) {
   check_matching_sizes("Binary function", "x", x, "y", y);
-  using int_vec_t = promote_scalar_t<value_type_t<T2>, plain_type_t<T1>>;
-  Eigen::Map<const int_vec_t> y_map(y.data(), y.size());
-#ifdef USE_STANC3
-  return x.binaryExpr(y_map, f);
-#else
-  return x.binaryExpr(y_map, f).eval();
-#endif
+  return make_holder(
+      [](auto& f_inner, auto& x_inner, auto& y_inner) {
+        using int_vec_t = promote_scalar_t<value_type_t<decltype(y_inner)>,
+                                           plain_type_t<decltype(x_inner)>>;
+        Eigen::Map<const int_vec_t> y_map(y_inner.data(), y_inner.size());
+        return x_inner.binaryExpr(y_map, f_inner);
+      },
+      std::forward<F>(f), std::forward<T1>(x), std::forward<T2>(y));
 }
 
 /**
- * Specialisation for use with a one-dimensional std::vector of integer types
+ * Specialization for use with a one-dimensional std::vector of integer types
  * and one Eigen vector (row or column).
  *
  * @tparam T1 Type of first argument to which functor is applied.
@@ -103,19 +104,20 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 template <typename T1, typename T2, typename F,
           require_std_vector_vt<std::is_integral, T1>* = nullptr,
           require_eigen_vector_vt<is_stan_scalar, T2>* = nullptr>
-inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
+inline auto apply_scalar_binary(T1&& x, T2&& y, F&& f) {
   check_matching_sizes("Binary function", "x", x, "y", y);
-  using int_vec_t = promote_scalar_t<value_type_t<T1>, plain_type_t<T2>>;
-  Eigen::Map<const int_vec_t> x_map(x.data(), x.size());
-#ifdef USE_STANC3
-  return x_map.binaryExpr(y, f);
-#else
-  return x_map.binaryExpr(y, f).eval();
-#endif
+  return make_holder(
+      [](auto& f_inner, auto& x_inner, auto& y_inner) {
+        using int_vec_t = promote_scalar_t<value_type_t<decltype(x_inner)>,
+                                           plain_type_t<decltype(y_inner)>>;
+        Eigen::Map<const int_vec_t> x_map(x_inner.data(), x_inner.size());
+        return x_map.binaryExpr(y_inner, f_inner);
+      },
+      std::forward<F>(f), std::forward<T1>(x), std::forward<T2>(y));
 }
 
 /**
- * Specialisation for use with one Eigen matrix and
+ * Specialization for use with one Eigen matrix and
  * a two-dimensional std::vector of integer types
  *
  * @tparam T1 Type of first argument to which functor is applied.
@@ -148,7 +150,7 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 }
 
 /**
- * Specialisation for use with a two-dimensional std::vector of integer types
+ * Specialization for use with a two-dimensional std::vector of integer types
  * and one Eigen matrix.
  *
  * @tparam T1 Type of first argument to which functor is applied.
@@ -181,7 +183,7 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 }
 
 /**
- * Specialisation for use when the first input is an Eigen type and the second
+ * Specialization for use when the first input is an Eigen type and the second
  * is a scalar. Eigen's unaryExpr framework is used for more efficient indexing
  * of both row- and column-major inputs. The unaryExpr framework also allows
  * for the scalar to be captured and applied to each element in the Eigen
@@ -194,22 +196,20 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
  * @param y Scalar input to which operation is applied.
  * @param f functor to apply to Eigen and scalar inputs.
  * @return Eigen object with result of applying functor to inputs.
- *
- * Note: The return expresssion needs to be evaluated, otherwise the captured
- *         function and scalar fall out of scope.
  */
 template <typename T1, typename T2, typename F, require_eigen_t<T1>* = nullptr,
           require_stan_scalar_t<T2>* = nullptr>
-inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
-#ifdef USE_STANC3
-  return x.unaryExpr([&f, y](const auto& v) { return f(v, y); });
-#else
-  return x.unaryExpr([&f, &y](const auto& v) { return f(v, y); }).eval();
-#endif
+inline auto apply_scalar_binary(T1&& x, T2&& y, F&& f) {
+  return make_holder(
+      [](auto& f_inner, auto& x_inner, auto& y_inner) {
+        return x_inner.unaryExpr(
+            [f_inner, y_inner](const auto& v) { return f_inner(v, y_inner); });
+      },
+      std::forward<F>(f), std::forward<T1>(x), std::forward<T2>(y));
 }
 
 /**
- * Specialisation for use when the first input is an scalar and the second is
+ * Specialization for use when the first input is an scalar and the second is
  * an Eigen type. Eigen's unaryExpr framework is used for more efficient
  * indexing of both row- and column-major inputs. The unaryExpr framework also
  * allows for the scalar to be captured and applied to each element in the
@@ -222,22 +222,20 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
  * @param y Eigen input to which operation is applied.
  * @param f functor to apply to Eigen and scalar inputs.
  * @return Eigen object with result of applying functor to inputs.
- *
- * Note: The return expresssion needs to be evaluated, otherwise the captured
- *         function and scalar fall out of scope.
  */
 template <typename T1, typename T2, typename F,
           require_stan_scalar_t<T1>* = nullptr, require_eigen_t<T2>* = nullptr>
-inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
-#ifdef USE_STANC3
-  return y.unaryExpr([&f, x](const auto& v) { return f(x, v); });
-#else
-  return y.unaryExpr([&f, &x](const auto& v) { return f(x, v); }).eval();
-#endif
+inline auto apply_scalar_binary(T1&& x, T2&& y, F&& f) {
+  return make_holder(
+      [](auto& f_inner, auto& x_inner, auto& y_inner) {
+        return y_inner.unaryExpr(
+            [f_inner, x_inner](const auto& v) { return f_inner(x_inner, v); });
+      },
+      std::forward<F>(f), std::forward<T1>(x), std::forward<T2>(y));
 }
 
 /**
- * Specialisation for use with (non-nested) std::vectors. Inputs are mapped
+ * Specialization for use with (non-nested) std::vectors. Inputs are mapped
  * to Eigen column vectors and then the result is evaluated directly into the
  * returned std::vector (via Eigen::Map).
  *
@@ -267,7 +265,7 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 }
 
 /**
- * Specialisation for use when the first input is a (non-nested) std::vector
+ * Specialization for use when the first input is a (non-nested) std::vector
  * and the second is a scalar. The std::vector input is mapped to an Eigen
  * column vector and then the result is evaluated directly into the returned
  * std::vector (via Eigen::Map).
@@ -292,12 +290,12 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
   using T_return = std::decay_t<decltype(f(x[0], y))>;
   std::vector<T_return> result(x.size());
   Eigen::Map<Eigen::Matrix<T_return, -1, 1>>(result.data(), result.size())
-      = x_vec.unaryExpr([&f, &y](const auto& v) { return f(v, y); });
+      = x_vec.unaryExpr([f, y](const auto& v) { return f(v, y); });
   return result;
 }
 
 /**
- * Specialisation for use when the first input is a scalar and the second is a
+ * Specialization for use when the first input is a scalar and the second is a
  * (non-nested) std::vector. The std::vector input is mapped to an Eigen
  * column vector and then the result is evaluated directly into the returned
  * std::vector (via Eigen::Map).
@@ -322,12 +320,12 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
   using T_return = std::decay_t<decltype(f(x, y[0]))>;
   std::vector<T_return> result(y.size());
   Eigen::Map<Eigen::Matrix<T_return, -1, 1>>(result.data(), result.size())
-      = y_vec.unaryExpr([&f, &x](const auto& v) { return f(x, v); });
+      = y_vec.unaryExpr([f, x](const auto& v) { return f(x, v); });
   return result;
 }
 
 /**
- * Specialisation for use with two nested containers (std::vectors).
+ * Specialization for use with two nested containers (std::vectors).
  * The returned scalar type is deduced to allow for cases where the input and
  * return scalar types differ (e.g., functions implicitly promoting
  * integers).
@@ -340,15 +338,12 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
  * @param f functor to apply to std::vector inputs.
  * @return std::vector with result of applying functor to inputs.
  */
-template <typename T1, typename T2, typename F,
-          require_all_std_vector_vt<is_container, T1, T2>* = nullptr>
+template <
+    typename T1, typename T2, typename F,
+    require_all_std_vector_vt<is_container_or_var_matrix, T1, T2>* = nullptr>
 inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
   check_matching_sizes("Binary function", "x", x, "y", y);
-#ifdef USE_STANC3
   using T_return = plain_type_t<decltype(apply_scalar_binary(x[0], y[0], f))>;
-#else
-  using T_return = decltype(apply_scalar_binary(x[0], y[0], f));
-#endif
   size_t y_size = y.size();
   std::vector<T_return> result(y_size);
   for (size_t i = 0; i < y_size; ++i) {
@@ -358,7 +353,7 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 }
 
 /**
- * Specialisation for use when the first input is a nested std::vector and the
+ * Specialization for use when the first input is a nested std::vector and the
  * second is a scalar. The returned scalar type is deduced to allow for cases
  * where the input and return scalar types differ (e.g., functions implicitly
  * promoting integers).
@@ -372,14 +367,10 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
  * @return std::vector with result of applying functor to inputs.
  */
 template <typename T1, typename T2, typename F,
-          require_std_vector_vt<is_container, T1>* = nullptr,
+          require_std_vector_vt<is_container_or_var_matrix, T1>* = nullptr,
           require_stan_scalar_t<T2>* = nullptr>
 inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
-#ifdef USE_STANC3
   using T_return = plain_type_t<decltype(apply_scalar_binary(x[0], y, f))>;
-#else
-  using T_return = decltype(apply_scalar_binary(x[0], y, f));
-#endif
   size_t x_size = x.size();
   std::vector<T_return> result(x_size);
   for (size_t i = 0; i < x_size; ++i) {
@@ -389,7 +380,7 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
 }
 
 /**
- * Specialisation for use when the first input is a scalar and the second is a
+ * Specialization for use when the first input is a scalar and the second is a
  * nested std::vector. The returned scalar type is deduced to allow for cases
  * where the input and return scalar types differ (e.g., functions implicitly
  * promoting integers).
@@ -404,13 +395,9 @@ inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
  */
 template <typename T1, typename T2, typename F,
           require_stan_scalar_t<T1>* = nullptr,
-          require_std_vector_vt<is_container, T2>* = nullptr>
+          require_std_vector_vt<is_container_or_var_matrix, T2>* = nullptr>
 inline auto apply_scalar_binary(const T1& x, const T2& y, const F& f) {
-#ifdef USE_STANC3
   using T_return = plain_type_t<decltype(apply_scalar_binary(x, y[0], f))>;
-#else
-  using T_return = decltype(apply_scalar_binary(x, y[0], f));
-#endif
   size_t y_size = y.size();
   std::vector<T_return> result(y_size);
   for (size_t i = 0; i < y_size; ++i) {
