@@ -15,7 +15,7 @@ inline int hypergeometric_rng(int N, int a, int b, RNG& rng) {
   using boost::variate_generator;
   using boost::math::hypergeometric_distribution;
   static const char* function = "hypergeometric_rng";
-  check_bounded(function, "Draws parameter", N, 0, a + b);
+  check_bounded(function, "Draws parameter", value_of(N), 0, a + b);
   check_positive(function, "Draws parameter", N);
   check_positive(function, "Successes in population parameter", a);
   check_positive(function, "Failures in population parameter", b);
